@@ -2,36 +2,39 @@
  * Copyright © 2008 Kristian Høgsberg
  * Copyright © 2013 Collabora, Ltd.
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting documentation, and
- * that the name of the copyright holders not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  The copyright holders make no representations
- * about the suitability of this software for any purpose.  It is provided "as
- * is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
- * OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include "config.h"
 
+#include <stdint.h>
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
+#include <errno.h>
 #include <cairo.h>
 
 #include <linux/input.h>
 
 #include "window.h"
-#include "scaler-client-protocol.h"
+#include "viewporter-client-protocol.h"
 
 #define BUFFER_SCALE 2
 static const int BUFFER_WIDTH = 421 * BUFFER_SCALE;
@@ -49,9 +52,8 @@ struct box {
 	struct widget *widget;
 	int width, height;
 
-	struct wl_scaler *scaler;
-	int scaler_version;
-	struct wl_viewport *viewport;
+	struct wp_viewporter *viewporter;
+	struct wp_viewport *viewport;
 
 	enum {
 		MODE_NO_VIEWPORT,
@@ -83,33 +85,30 @@ set_my_viewport(struct box *box)
 	src_width = wl_fixed_from_double((RECT_W - 0.5) / BUFFER_SCALE);
 	src_height = wl_fixed_from_double((RECT_H - 0.5) / BUFFER_SCALE);
 
-	if (box->scaler_version < 2 && box->mode != MODE_SRC_DST) {
-		fprintf(stderr, "Error: server's wl_scaler interface version "
-			"%d does not support this mode.\n",
-			box->scaler_version);
-		exit(1);
-	}
-
 	switch (box->mode){
 	case MODE_SRC_ONLY:
-		wl_viewport_set_source(box->viewport, src_x, src_y,
+		/* In SRC_ONLY mode we're just cropping - in order
+		 * for the surface size to remain an integer, the
+		 * compositor will generate an error if we use a
+		 * fractional width or height.
+		 *
+		 * We use fractional width/height for the other cases
+		 * to ensure fractional values are still tested.
+		 */
+		src_width = wl_fixed_from_int(RECT_W / BUFFER_SCALE);
+		src_height = wl_fixed_from_int(RECT_H / BUFFER_SCALE);
+		wp_viewport_set_source(box->viewport, src_x, src_y,
 				       src_width, src_height);
 		break;
 	case MODE_DST_ONLY:
-		wl_viewport_set_destination(box->viewport,
+		wp_viewport_set_destination(box->viewport,
 					    dst_width, dst_height);
 		break;
 	case MODE_SRC_DST:
-		if (box->scaler_version < 2) {
-			wl_viewport_set(box->viewport,
-					src_x, src_y, src_width, src_height,
-					dst_width, dst_height);
-		} else {
-			wl_viewport_set_source(box->viewport, src_x, src_y,
-					       src_width, src_height);
-			wl_viewport_set_destination(box->viewport,
-						    dst_width, dst_height);
-		}
+		wp_viewport_set_source(box->viewport, src_x, src_y,
+				       src_width, src_height);
+		wp_viewport_set_destination(box->viewport,
+					    dst_width, dst_height);
 		break;
 	default:
 		assert(!"not reached");
@@ -122,7 +121,7 @@ resize_handler(struct widget *widget,
 {
 	struct box *box = data;
 
-	/* Dont resize me */
+	/* Don't resize me */
 	widget_set_size(box->widget, box->width, box->height);
 }
 
@@ -187,14 +186,11 @@ global_handler(struct display *display, uint32_t name,
 {
 	struct box *box = data;
 
-	if (strcmp(interface, "wl_scaler") == 0) {
-		box->scaler_version = version < 2 ? version : 2;
+	if (strcmp(interface, "wp_viewporter") == 0) {
+		box->viewporter = display_bind(display, name,
+					       &wp_viewporter_interface, 1);
 
-		box->scaler = display_bind(display, name,
-					   &wl_scaler_interface,
-					   box->scaler_version);
-
-		box->viewport = wl_scaler_get_viewport(box->scaler,
+		box->viewport = wp_viewporter_get_viewport(box->viewporter,
 			widget_get_wl_surface(box->widget));
 
 		set_my_viewport(box);
@@ -274,12 +270,6 @@ main(int argc, char *argv[])
 	struct timeval tv;
 	int i;
 
-	d = display_create(&argc, argv);
-	if (d == NULL) {
-		fprintf(stderr, "failed to create display: %m\n");
-		return -1;
-	}
-
 	box.mode = MODE_SRC_DST;
 
 	for (i = 1; i < argc; i++) {
@@ -297,6 +287,13 @@ main(int argc, char *argv[])
 		}
 	}
 
+	d = display_create(&argc, argv);
+	if (d == NULL) {
+		fprintf(stderr, "failed to create display: %s\n",
+			strerror(errno));
+		return -1;
+	}
+
 	gettimeofday(&tv, NULL);
 	srandom(tv.tv_usec);
 
@@ -306,6 +303,7 @@ main(int argc, char *argv[])
 	box.window = window_create(d);
 	box.widget = window_add_widget(box.window, &box);
 	window_set_title(box.window, "Scaler Test Box");
+	window_set_appid(box.window, "org.freedesktop.weston.scaler-test-box");
 	window_set_buffer_scale(box.window, BUFFER_SCALE);
 
 	widget_set_resize_handler(box.widget, resize_handler);
@@ -319,8 +317,16 @@ main(int argc, char *argv[])
 	display_set_user_data(box.display, &box);
 	display_set_global_handler(box.display, global_handler);
 
+	if (box.mode != MODE_NO_VIEWPORT && !box.viewport) {
+		fprintf(stderr, "compositor doesn't support viewporter\n");
+		return -1;
+	}
+
 	display_run(d);
 
+	widget_destroy(box.widget);
 	window_destroy(box.window);
+	display_destroy(d);
+
 	return 0;
 }
