@@ -32,6 +32,7 @@
 #include <cairo.h>
 #include <math.h>
 #include <assert.h>
+#include <errno.h>
 
 #include <linux/input.h>
 #include <wayland-client.h>
@@ -43,7 +44,7 @@
 
 #include "shared/helpers.h"
 #include "shared/xalloc.h"
-#include "shared/zalloc.h"
+#include <libweston/zalloc.h>
 #include "window.h"
 
 #if 0
@@ -55,8 +56,8 @@
 
 static int32_t option_red_mode;
 static int32_t option_triangle_mode;
-static int32_t option_no_triangle;
-static int32_t option_help;
+static bool option_no_triangle;
+static bool option_help;
 
 static const struct weston_option options[] = {
 	{ WESTON_OPTION_INTEGER, "red-mode", 'r', &option_red_mode },
@@ -295,7 +296,7 @@ create_shader(const char *source, GLenum shader_type)
 		char log[1000];
 		GLsizei len;
 		glGetShaderInfoLog(shader, 1000, &len, log);
-		fprintf(stderr, "Error: compiling %s: %*s\n",
+		fprintf(stderr, "Error: compiling %s: %.*s\n",
 			shader_type == GL_VERTEX_SHADER ? "vertex" : "fragment",
 			len, log);
 		exit(1);
@@ -324,7 +325,7 @@ triangle_init_gl(struct triangle_gl_state *trigl)
 		char log[1000];
 		GLsizei len;
 		glGetProgramInfoLog(program, 1000, &len, log);
-		fprintf(stderr, "Error: linking:\n%*s\n", len, log);
+		fprintf(stderr, "Error: linking:\n%.*s\n", len, log);
 		exit(1);
 	}
 
@@ -518,7 +519,8 @@ static void
 triangle_destroy(struct triangle *tri)
 {
 	if (tri->egl_surface)
-		eglDestroySurface(tri->egl->dpy, tri->egl_surface);
+		weston_platform_destroy_egl_surface(tri->egl->dpy,
+						    tri->egl_surface);
 
 	if (tri->egl_window)
 		wl_egl_window_destroy(tri->egl_window);
@@ -731,6 +733,8 @@ demoapp_create(struct display *display)
 	app->window = window_create(app->display);
 	app->widget = window_frame_create(app->window, app);
 	window_set_title(app->window, "Wayland Sub-surface Demo");
+	window_set_appid(app->window,
+			 "org.freedesktop.weston.wayland-sub-surface-demo");
 
 	window_set_key_handler(app->window, key_handler);
 	window_set_user_data(app->window, app);
@@ -787,7 +791,8 @@ main(int argc, char *argv[])
 
 	display = display_create(&argc, argv);
 	if (display == NULL) {
-		fprintf(stderr, "failed to create display: %m\n");
+		fprintf(stderr, "failed to create display: %s\n",
+			strerror(errno));
 		return -1;
 	}
 
