@@ -40,6 +40,8 @@ struct widget;
 struct display;
 struct input;
 struct output;
+struct tablet;
+struct tablet_tool;
 
 struct task {
 	void (*run)(struct task *task, uint32_t events);
@@ -71,9 +73,6 @@ display_get_display(struct display *display);
 int
 display_has_subcompositor(struct display *display);
 
-cairo_device_t *
-display_get_cairo_device(struct display *display);
-
 struct wl_compositor *
 display_get_compositor(struct display *display);
 
@@ -114,22 +113,6 @@ display_set_output_configure_handler(struct display *display,
 struct wl_data_source *
 display_create_data_source(struct display *display);
 
-#ifdef EGL_NO_DISPLAY
-EGLDisplay
-display_get_egl_display(struct display *d);
-
-EGLConfig
-display_get_argb_egl_config(struct display *d);
-
-int
-display_acquire_window_surface(struct display *display,
-			       struct window *window,
-			       EGLContext ctx);
-void
-display_release_window_surface(struct display *display,
-			       struct window *window);
-#endif
-
 #define SURFACE_OPAQUE 0x01
 #define SURFACE_SHM    0x02
 
@@ -284,6 +267,57 @@ typedef void (*widget_axis_handler_t)(struct widget *widget,
 				      uint32_t axis,
 				      wl_fixed_t value,
 				      void *data);
+typedef int (*widget_tablet_tool_motion_handler_t)(struct widget *widget,
+						   struct tablet_tool *tool,
+						   float x, float y,
+						   void *data);
+typedef void (*widget_tablet_tool_down_handler_t)(struct widget *widget,
+						  struct tablet_tool *tool,
+						  void *data);
+typedef void (*widget_tablet_tool_up_handler_t)(struct widget *widget,
+						struct tablet_tool *tool,
+						void *data);
+typedef void (*widget_tablet_tool_pressure_handler_t)(struct widget *widget,
+						      struct tablet_tool *tool,
+						      uint32_t pressure,
+						      void *data);
+typedef void (*widget_tablet_tool_distance_handler_t)(struct widget *widget,
+						      struct tablet_tool *tool,
+						      uint32_t distance,
+						      void *data);
+typedef void (*widget_tablet_tool_tilt_handler_t)(struct widget *widget,
+						  struct tablet_tool *tool,
+						  int32_t tilt_x, int32_t tilt_y,
+						  void *data);
+typedef void (*widget_tablet_tool_rotation_handler_t)(struct widget *widget,
+						      struct tablet_tool *tool,
+						      int32_t rotation,
+						      void *data);
+typedef void (*widget_tablet_tool_slider_handler_t)(struct widget *widget,
+						    struct tablet_tool *tool,
+						    int32_t slider,
+						    void *data);
+typedef void (*widget_tablet_tool_wheel_handler_t)(struct widget *widget,
+						   struct tablet_tool *tool,
+						   wl_fixed_t degrees,
+						   int32_t clicks,
+						   void *data);
+typedef void (*widget_tablet_tool_proximity_in_handler_t)(struct widget *widget,
+							  struct tablet_tool *tool,
+							  struct tablet *tablet,
+							  void *data);
+typedef void (*widget_tablet_tool_proximity_out_handler_t)(struct widget *widget,
+							   struct tablet_tool *tool,
+							   void *data);
+typedef void (*widget_tablet_tool_button_handler_t)(struct widget *widget,
+						    struct tablet_tool *tool,
+						    uint32_t button,
+						    uint32_t state,
+						    void *data);
+typedef void (*widget_tablet_tool_frame_handler_t)(struct widget *widget,
+						   struct tablet_tool *tool,
+						   uint32_t time,
+						   void *data);
 
 typedef void (*widget_pointer_frame_handler_t)(struct widget *widget,
 					       struct input *input,
@@ -416,7 +450,6 @@ struct wl_subsurface *
 widget_get_wl_subsurface(struct widget *widget);
 
 enum window_buffer_type {
-	WINDOW_BUFFER_TYPE_EGL_WINDOW,
 	WINDOW_BUFFER_TYPE_SHM,
 };
 
@@ -512,6 +545,28 @@ window_get_appid(struct window *window);
 void
 window_set_text_cursor_position(struct window *window, int32_t x, int32_t y);
 
+enum render_intent {
+	RENDER_INTENT_PERCEPTUAL,
+	RENDER_INTENT_RELATIVE,
+	RENDER_INTENT_RELATIVE_BPC,
+	RENDER_INTENT_SATURATION,
+	RENDER_INTENT_ABSOLUTE,
+};
+
+struct render_intent_info {
+	enum render_intent intent;
+	uint32_t protocol_intent;
+	const char *desc;
+};
+
+const struct render_intent_info *
+render_intent_info_from(enum render_intent intent);
+
+bool
+widget_set_image_description_icc(struct widget *widget, int icc_fd,
+				 uint32_t length, uint32_t offset,
+				 enum render_intent intent, char **err_msg);
+
 int
 widget_set_tooltip(struct widget *parent, char *entry, float x, float y);
 
@@ -526,6 +581,8 @@ widget_destroy(struct widget *widget);
 void
 widget_set_default_cursor(struct widget *widget, int cursor);
 void
+widget_set_default_tablet_cursor(struct widget *widget, int cursor);
+void
 widget_get_allocation(struct widget *widget, struct rectangle *allocation);
 
 void
@@ -598,6 +655,31 @@ widget_set_axis_handlers(struct widget *widget,
 			widget_axis_source_handler_t axis_source_handler,
 			widget_axis_stop_handler_t axis_stop_handler,
 			widget_axis_discrete_handler_t axis_discrete_handler);
+void
+widget_set_tablet_tool_axis_handlers(struct widget *widget,
+				     widget_tablet_tool_motion_handler_t motion,
+				     widget_tablet_tool_pressure_handler_t pressure,
+				     widget_tablet_tool_distance_handler_t distance,
+				     widget_tablet_tool_tilt_handler_t tilt,
+				     widget_tablet_tool_rotation_handler_t rotation,
+				     widget_tablet_tool_slider_handler_t slider,
+				     widget_tablet_tool_wheel_handler_t wheel);
+void
+widget_set_tablet_tool_up_handler(struct widget *widget,
+				  widget_tablet_tool_up_handler_t handler);
+void
+widget_set_tablet_tool_down_handler(struct widget *widget,
+				    widget_tablet_tool_down_handler_t handler);
+void
+widget_set_tablet_tool_proximity_handlers(struct widget *widget,
+					  widget_tablet_tool_proximity_in_handler_t in_handler,
+					  widget_tablet_tool_proximity_out_handler_t out_handler);
+void
+widget_set_tablet_tool_button_handler(struct widget *widget,
+				      widget_tablet_tool_button_handler_t handler);
+void
+widget_set_tablet_tool_frame_handler(struct widget *widget,
+				     widget_tablet_tool_frame_handler_t handler);
 
 void
 window_inhibit_redraw(struct window *window);
@@ -721,6 +803,18 @@ xkb_mod_mask_t
 keysym_modifiers_get_mask(struct wl_array *modifiers_map,
 			  const char *name);
 
+uint32_t
+tablet_tool_get_type(struct tablet_tool *tool);
+
+uint64_t
+tablet_tool_get_serial(struct tablet_tool *tool);
+
+uint64_t
+tablet_tool_get_hwid(struct tablet_tool *tool);
+
+void
+tablet_tool_set_cursor_image(struct tablet_tool *tool, int cursor);
+
 struct toytimer;
 typedef void (*toytimer_cb)(struct toytimer *);
 
