@@ -47,11 +47,14 @@
 #include "shared/weston-egl-ext.h"
 #include "shared/cairo-util.h"
 #include "shared/xalloc.h"
+#include "shared/timespec-util.h"
 #include "linux-dmabuf.h"
 #include "output-capture.h"
 #include "presentation-time-server-protocol.h"
 #include <libweston/windowed-output-api.h>
 
+#define DEFAULT_OUTPUT_REPAINT_REFRESH 60000 /* In mHz. */
+
 struct headless_backend {
 	struct weston_backend base;
 	struct weston_compositor *compositor;
@@ -63,6 +66,9 @@ struct headless_backend {
 
 	const struct pixel_format_info **formats;
 	unsigned int formats_count;
+
+	int refresh;
+	bool repaint_only_on_capture;
 };
 
 struct headless_head {
@@ -131,10 +137,8 @@ static int
 finish_frame_handler(void *data)
 {
 	struct headless_output *output = data;
-	struct timespec ts;
 
-	weston_compositor_read_presentation_clock(output->base.compositor, &ts);
-	weston_output_finish_frame(&output->base, &ts, 0);
+	weston_output_finish_frame_from_timer(&output->base);
 
 	return 1;
 }
@@ -157,6 +161,7 @@ headless_output_repaint(struct weston_output *output_base)
 	struct headless_output *output = to_headless_output(output_base);
 	struct weston_compositor *ec;
 	pixman_region32_t damage;
+	int delay_msec;
 
 	assert(output);
 
@@ -173,7 +178,8 @@ headless_output_repaint(struct weston_output *output_base)
 
 	pixman_region32_fini(&damage);
 
-	wl_event_source_timer_update(output->finish_frame_timer, 16);
+	delay_msec = millihz_to_nsec(output->mode.refresh) / 1000000;
+	wl_event_source_timer_update(output->finish_frame_timer, delay_msec);
 
 	return 0;
 }
@@ -399,7 +405,7 @@ headless_output_set_size(struct weston_output *base,
 	assert(!output->base.current_mode);
 
 	/* Make sure we have scale set. */
-	assert(output->base.scale);
+	assert(output->base.current_scale);
 
 	wl_list_for_each(head, &output->base.head_list, output_link) {
 		weston_head_set_monitor_strings(head, "weston", "headless",
@@ -409,14 +415,14 @@ headless_output_set_size(struct weston_output *base,
 		weston_head_set_physical_size(head, width, height);
 	}
 
-	output_width = width * output->base.scale;
-	output_height = height * output->base.scale;
+	output_width = width * output->base.current_scale;
+	output_height = height * output->base.current_scale;
 
 	output->mode.flags =
 		WL_OUTPUT_MODE_CURRENT | WL_OUTPUT_MODE_PREFERRED;
 	output->mode.width = output_width;
 	output->mode.height = output_height;
-	output->mode.refresh = 60000;
+	output->mode.refresh = output->backend->refresh;
 	wl_list_insert(&output->base.mode_list, &output->mode.link);
 
 	output->base.current_mode = &output->mode;
@@ -451,6 +457,7 @@ headless_output_create(struct weston_backend *backend, const char *name)
 	output->base.disable = headless_output_disable;
 	output->base.enable = headless_output_enable;
 	output->base.attach_head = NULL;
+	output->base.repaint_only_on_capture = b->repaint_only_on_capture;
 
 	output->backend = b;
 
@@ -480,6 +487,8 @@ headless_head_create(struct weston_backend *base,
 	weston_head_set_connection_status(&head->base, true);
 	weston_head_set_supported_eotf_mask(&head->base,
 					    WESTON_EOTF_MODE_ALL_MASK);
+	weston_head_set_supported_colorimetry_mask(&head->base,
+						   WESTON_COLORIMETRY_MODE_ALL_MASK);
 
 	/* Ideally all attributes of the head would be set here, so that the
 	 * user has all the information when deciding to create outputs.
@@ -565,6 +574,18 @@ headless_backend_create(struct weston_compositor *compositor,
 	b->formats_count = ARRAY_LENGTH(headless_formats);
 	b->formats = pixel_format_get_array(headless_formats, b->formats_count);
 
+	/* Wayland event source's timeout has a granularity of the order of
+	 * milliseconds so the highest supported rate is 1 kHz. 0 is a special
+	 * value that enables repaints only on capture. */
+	if (config->refresh > 0) {
+		b->refresh = MIN(config->refresh, 1000000);
+	} else if (config->refresh == 0) {
+		b->refresh = 1000000;
+		b->repaint_only_on_capture = true;
+	} else {
+		b->refresh = DEFAULT_OUTPUT_REPAINT_REFRESH;
+	}
+
 	if (!compositor->renderer) {
 		switch (config->renderer) {
 		case WESTON_RENDERER_GL: {
@@ -598,26 +619,21 @@ headless_backend_create(struct weston_compositor *compositor,
 			break;
 		default:
 			weston_log("Error: unsupported renderer\n");
+			ret = -1;
 			break;
 		}
 
 		if (ret < 0)
 			goto err_input;
 
-		if (compositor->renderer->import_dmabuf) {
-			if (linux_dmabuf_setup(compositor) < 0) {
-				weston_log("Error: dmabuf protocol setup failed.\n");
-				goto err_input;
-			}
-		}
-
 		/* Support zwp_linux_explicit_synchronization_unstable_v1 to enable
 		 * testing. */
 		if (linux_explicit_synchronization_setup(compositor) < 0)
 			goto err_input;
 	}
 
-	ret = weston_plugin_api_register(compositor, WESTON_WINDOWED_OUTPUT_API_NAME,
+	ret = weston_plugin_api_register(compositor,
+					 WESTON_WINDOWED_OUTPUT_API_NAME_HEADLESS,
 					 &api, sizeof(api));
 
 	if (ret < 0) {
@@ -639,6 +655,7 @@ err_free:
 static void
 config_init_to_defaults(struct weston_headless_backend_config *config)
 {
+	config->refresh = DEFAULT_OUTPUT_REPAINT_REFRESH;
 }
 
 WL_EXPORT int
