@@ -493,27 +493,48 @@ frame_resize(struct frame *frame, int32_t width, int32_t height)
 }
 
 void
-frame_resize_inside(struct frame *frame, int32_t width, int32_t height)
+frame_border_sizes(struct frame *frame, int32_t *top, int32_t *bottom,
+		   int32_t *left, int32_t *right)
 {
 	struct theme *t = frame->theme;
-	int decoration_width, decoration_height, titlebar_height;
 
+	/* Top may have a titlebar */
 	if (frame->title || !wl_list_empty(&frame->buttons))
-		titlebar_height = t->titlebar_height;
+		*top = t->titlebar_height;
 	else
-		titlebar_height = t->width;
+		*top = t->width;
 
-	if (frame->flags & FRAME_FLAG_MAXIMIZED) {
-		decoration_width = t->width * 2;
-		decoration_height = t->width + titlebar_height;
-	} else {
-		decoration_width = (t->width + t->margin) * 2;
-		decoration_height = t->width +
-			titlebar_height + t->margin * 2;
-	}
+	/* All other sides have the basic frame thickness */
+	*bottom = t->width;
+	*right = t->width;
+	*left = t->width;
+}
+void
+frame_decoration_sizes(struct frame *frame, int32_t *top, int32_t *bottom,
+		       int32_t *left, int32_t *right)
+{
+	struct theme *t = frame->theme;
 
-	frame_resize(frame, width + decoration_width,
-		     height + decoration_height);
+	frame_border_sizes(frame, top, bottom, left, right);
+
+	if (frame->flags & FRAME_FLAG_MAXIMIZED)
+		return;
+
+	/* Not maximized, add shadows */
+	*top += t->margin;
+	*bottom += t->margin;
+	*left += t->margin;
+	*right += t->margin;
+}
+
+void
+frame_resize_inside(struct frame *frame, int32_t width, int32_t height)
+{
+	int32_t top, bottom, left, right;
+
+	frame_decoration_sizes(frame, &top, &bottom, &left, &right);
+	frame_resize(frame, width + left + right,
+		     height + top + bottom);
 }
 
 int32_t
@@ -994,6 +1015,44 @@ frame_double_touch_up(struct frame *frame, void *data, int32_t id)
 	}
 }
 
+enum theme_location
+frame_tablet_tool_motion(struct frame *frame, void *data, int x, int y)
+{
+	struct frame_pointer *tool_pointer = frame_pointer_get(frame, data);
+	struct frame_button *button,
+			    *prev_button = tool_pointer->hover_button;
+	enum theme_location location;
+
+	location = theme_get_location(frame->theme, tool_pointer->x,
+				      tool_pointer->y, frame->width,
+				      frame->height,
+				      frame->flags & FRAME_FLAG_MAXIMIZED ?
+				      THEME_FRAME_MAXIMIZED : 0);
+
+	if (!tool_pointer)
+		return location;
+
+	tool_pointer->x = x;
+	tool_pointer->y = y;
+
+	button = frame_find_button(frame, x, y);
+
+	if (prev_button) {
+		if (prev_button == button)
+			/* The button hasn't changed so we're done here */
+			return location;
+		else
+			frame_button_leave(prev_button, tool_pointer);
+	}
+
+	if (button)
+		frame_button_enter(button);
+
+	tool_pointer->hover_button = button;
+
+	return location;
+}
+
 void
 frame_repaint(struct frame *frame, cairo_t *cr)
 {
